//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rargs.cxx
// Purpose:    Argument list of literal in a rule.
// Last Change:27.09.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "rvars.h"
#include "rarg.h"

#include "rargs.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

rargs_c::rargs_c(rarg_t head)
{
	// Check Parameters:
	CHECK(head != 0, "rargs_c::Constructor: Head is NIL");
	CHECK(head->valid(), "rargs_c::Constructor: Head is invalid");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = RARGS_MAGIC;
#endif

	// Initialize object:
	Head = head;
	Tail = 0;
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

rargs_c::~rargs_c(void)
{
#ifdef	VER_DEBUG
	Magic = 0;
#endif
}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	rargs_c::valid(void) const
{
	// Check object pointer:
	CHECK(this != 0, "rargs_c::valid: NIL object");

	// Magic number must be correct:
	if(Magic != RARGS_MAGIC)
		return(BOOL_FALSE);

	// Head must be not null:
	if(Head == 0)
		return(BOOL_FALSE);

	// Head must be valid:
	if(Head->valid() != BOOL_TRUE)
		return(BOOL_FALSE);

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// Print this term list:
//-----------------------------------------------------------------------------

void rargs_c::print(rvars_t vars) const
{
	rargs_c *p;

	// Check Consistency:
	CHECK(valid(), "rargs_c::print: Bad object");

	// Print list iteratively:
	Head->print(vars);
	p = Tail;
	while(p != 0) {
		out_c::print_str(", ");
		p->get_head()->print(vars);
		p = p->get_tail();
	}
}

